/*--------------------------------------------------------------------------*/
/* A9139F6.H                                                                  */
/*                                                                          */
/* Header file for A9139F6 RF SoC microcontroller.                            */
/* Copyright (c) 2013 AMICCOM Electronics Corp.                             */
/*                                                                          */
/* All rights reserved.                                                     */
/*--------------------------------------------------------------------------*/
#ifndef _A9139F6_H_
#define _A9139F6_H_
//----------------------------------------------------------------------------
// 8051 SFR
//----------------------------------------------------------------------------
/*  BYTE Registers  */
sfr P0      = 0x80;
sfr P1      = 0x90;
sfr P2      = 0xA0;
sfr P3      = 0xB0;
sfr PSW     = 0xD0;
sfr ACC     = 0xE0;
sfr B       = 0xF0;
sfr SP      = 0x81;
sfr DPL     = 0x82;
sfr DPH     = 0x83;
sfr DPL1    = 0x84;
sfr DPH1    = 0x85;
sfr PCON    = 0x87;
sfr TCON    = 0x88;
sfr TMOD    = 0x89;
sfr TL0     = 0x8A;
sfr TL1     = 0x8B;
sfr TH0     = 0x8C;
sfr TH1     = 0x8D;
sfr CKCON   = 0x8E;
sfr EIF     = 0x91;
sfr SCON    = 0x98;
sfr SCON1	= 0xC0;
sfr SBUF    = 0x99;
sfr SBUF1	= 0xC1;
sfr FLASHCTRL = 0x9A;
sfr FLASHTMR = 0x9B;
sfr FLASHPT0 = 0xA4;
sfr FLASHPT1 = 0xA5;
sfr IE      = 0xA8;
sfr PWM0CON = 0xA9;
sfr PWM0H   = 0xAA;
sfr PWM0L   = 0xAB;
sfr PWM1CON = 0xB1;
sfr PWM1H   = 0xB2;
sfr PWM1L   = 0xB3;

sfr PWM2CON = 0xAC;
sfr PWM2H   = 0xAD;
sfr PWM2L   = 0xAE;

sfr PWM3CON = 0xB4;
sfr PWM3H   = 0xB5;
sfr PWM3L   = 0xB6;

sfr IP      = 0xB8;
sfr PCONE   = 0xB9;
sfr RSFLAG  = 0xBA;
sfr IOSEL   = 0xBB;
sfr ADCCH   = 0xBC;	

sfr ODSEL   = 0xBE;
sfr T2CON   = 0xC8;
sfr T2IF    = 0xC9;
sfr RCAP2L  = 0xCA;
sfr RCAP2H  = 0xCB;
sfr TL2     = 0xCC;
sfr TH2     = 0xCD;
sfr WDCON   = 0xD8;
sfr EIE     = 0xE8;
sfr TA      = 0xEB;
sfr SPCR    = 0xEC;
sfr SPSR    = 0xED;
sfr SPDR    = 0xEE;
sfr SSCR    = 0xEF;
sfr I2CSADR = 0xF1;
sfr I2CSCR  = 0xF2;
sfr I2CSBUF = 0xF3;
sfr I2CMSA  = 0xF4;
sfr I2CMCR  = 0xF5;
sfr I2CMBUF = 0xF6;
sfr I2CMTP  = 0xF7;
sfr EIP     = 0xF8;

sfr P0OE    = 0xD1;
sfr P0PUN   = 0xD2;
sfr P0WUN   = 0xD3;
sfr P1OE    = 0xD9;
sfr P1PUN   = 0xDA;
sfr P1WUN   = 0xDB;
sfr P2OE    = 0xA1;
sfr P2PUN   = 0xA2;
sfr P2WUN   = 0xA3;
sfr P3OE    = 0xE1;
sfr P3PUN   = 0xE2;
sfr P3WUN   = 0xE3;

sfr SBRG0H   = 0x9E;
sfr SBRG0L   = 0x9F;

sfr SBRG1H   = 0xA6;
sfr SBRG1L   = 0xA7;



// Timer3 / Timer4
sfr ETMR	= 0xFA;
sfr T3CON	= 0xFB;
sfr RLDL3	= 0xFC;
sfr RLDH3	= 0xFD;
sfr TL3		= 0xFE;
sfr TH3		= 0xFF;

sfr T4CON	= 0xC3;
sfr RLDL4	= 0xC4;
sfr RLDH4	= 0xC5;
sfr TL4		= 0xC6;
sfr TH4		= 0xC7;


/*  BIT Registers  */
/*  PSW  */
sbit CY     = PSW^7;
sbit AC     = PSW^6;
sbit F0     = PSW^5;
sbit RS1    = PSW^4;
sbit RS0    = PSW^3;
sbit OV     = PSW^2;
sbit P      = PSW^0;

/*  TCON  */
sbit TF1    = TCON^7;
sbit TR1    = TCON^6;
sbit TF0    = TCON^5;
sbit TR0    = TCON^4;
sbit IE1    = TCON^3;
sbit IT1    = TCON^2;
sbit IE0    = TCON^1;
sbit IT0    = TCON^0;

/*  IE  */
sbit EA     = IE^7;
sbit ES1	= IE^6;
sbit ET2    = IE^5;
sbit ES     = IE^4;
sbit ET1    = IE^3;
sbit EX1    = IE^2;
sbit ET0    = IE^1;
sbit EX0    = IE^0;

/*  IP  */
sbit PT2    = IP^5;
sbit PS     = IP^4;
sbit PT1    = IP^3;
sbit PX1    = IP^2;
sbit PT0    = IP^1;
sbit PX0    = IP^0;

/*  P3  */
sbit RD     = P3^7;
sbit WR     = P3^6;
sbit T1     = P3^5;
sbit T0     = P3^4;
sbit INT1   = P3^3;
sbit INT0   = P3^2;
sbit TXD    = P3^1;
sbit RXD    = P3^0;

/*  SCON  */
sbit SM0    = SCON^7;
sbit SM1    = SCON^6;
sbit SM2    = SCON^5;
sbit REN    = SCON^4;
sbit TB8    = SCON^3;
sbit RB8    = SCON^2;
sbit TI     = SCON^1;
sbit RI     = SCON^0;

/*  SCON1  */
sbit SM10    = SCON1^7;
sbit SM11    = SCON1^6;
sbit SM12    = SCON1^5;
sbit REN1    = SCON1^4;
sbit TB18    = SCON1^3;
sbit RB18    = SCON1^2;
sbit TI1     = SCON1^1;
sbit RI1     = SCON1^0;

/*  P1  */
sbit T2EX   = P1^1;
sbit T2     = P1^0;

/*  T2CON  */
sbit TF2    = T2CON^7;
sbit EXF2   = T2CON^6;
sbit RCLK   = T2CON^5;
sbit TCLK   = T2CON^4;
sbit EXEN2  = T2CON^3;
sbit TR2    = T2CON^2;
sbit C_T2   = T2CON^1;
sbit CP_RL2 = T2CON^0;

/*  P0  */
sbit P0_7   = P0^7;
sbit P0_6   = P0^6;
sbit P0_5   = P0^5;
sbit P0_4   = P0^4;
sbit P0_3   = P0^3;
sbit P0_2   = P0^2;
sbit P0_1   = P0^1;
sbit P0_0   = P0^0;

/*  P1  */
sbit P1_7   = P1^7;
sbit P1_6   = P1^6;
sbit P1_5   = P1^5;
sbit P1_4   = P1^4;
sbit P1_3   = P1^3;
sbit P1_2   = P1^2;
sbit P1_1   = P1^1;
sbit P1_0   = P1^0;

/*  P2  */
sbit P2_7   = P2^7;
sbit P2_6   = P2^6;
sbit P2_5   = P2^5;
sbit P2_4   = P2^4;
sbit P2_3   = P2^3;
sbit P2_2   = P2^2;
sbit P2_1   = P2^1;
sbit P2_0   = P2^0;

/*  P3  */
sbit P3_7   = P3^7;
sbit P3_6   = P3^6;
sbit P3_5   = P3^5;
sbit P3_4   = P3^4;
sbit P3_3   = P3^3;
sbit P3_2   = P3^2;
sbit P3_1   = P3^1;
sbit P3_0   = P3^0;

/* EIE */
sbit EX2    = EIE^0;
sbit EX3    = EIE^1;
sbit EX4    = EIE^2;
sbit ERFINT = EIE^3;
sbit EX5    = EIE^3;
sbit EX6    = EIE^4;	//	  EKEYINT
sbit EWDI   = EIE^5;
sbit EI2CM  = EIE^6;
sbit EI2CS  = EIE^7;
sbit ESPI   = EIE^7;

//--------------------------------------------------------------------------
// RF Register
//--------------------------------------------------------------------------
#define CONFIG          	0x2000
#define RSTCTL_REG      	0x2000
#define MODEC1_REG      	0x2001
#define MODEC2_REG      	0x2002
#define MODEC3_REG      	0x2003
#define CAC1_REG        	0x2004
#define CAC2_REG        	0x2005
#define FIFO1_REG       	0x2006
#define FIFO2_REG       	0x2007
#define RCOSC1_REG      	0x2008
#define RCOSC2_REG      	0x2009
#define RCOSC3_REG      	0x200A
#define RCOSC4_REG      	0x200B
#define RCOSC5_REG      	0x200C
#define SYCK1_REG       	0x200D
#define SYCK2_REG       	0x200E
#define PLL1_REG        	0x200F
#define PLL2_REG        	0x2010
#define PLL3_REG        	0x2011
#define PLL4_REG        	0x2012
#define PLL5_REG        	0x2013
#define PLL6_REG        	0x2014
#define PLL7_REG        	0x2015
#define PLL8_REG        	0x2016
#define PLL9_REG        	0x2017
#define PLL10_REG       	0x2018
#define PLL11_REG       	0x2019
#define PLL12_REG       	0x201A
#define CLOCK_REG       	0x201B
#define TX1_REG         	0x201C
#define TX2_REG         	0x201D
#define TX3_REG         	0x201E
#define TX4_REG         	0x201F
#define WOR1_REG        	0x2020
#define WOR2_REG        	0x2021
#define WOR3_REG        	0x2022
#define WOR4_REG        	0x2023
#define RFI1_REG        	0x2024
#define RFI2_REG        	0x2025
#define PM1_REG         	0x2026
#define PM2_REG         	0x2027
#define RTH1_REG        	0x2028
#define RTH2_REG        	0x2029
#define AGC1_REG        	0x202A
#define AGC2_REG        	0x202B
#define AGC3_REG        	0x202C
#define AGC4_REG        	0x202D
#define CKO1_REG        	0x202E
#define CKO2_REG        	0x202F
#define GPIO1_REG       	0x2030
#define GPIO2_REG       	0x2031
#define PIN1_REG        	0x2032
#define PIN2_REG        	0x2033
#define VCB1_REG        	0x2034
#define VCB2_REG        	0x2035
#define CHG1_REG        	0x2036
#define CHG2_REG        	0x2037
#define CHG3_REG        	0x2038
#define CHG4_REG        	0x2039
#define IF1_REG         	0x203A
#define IF2_REG         	0x203B
#define IF3_REG         	0x203C
#define IF4_REG         	0x203D
#define ACK1_REG        	0x203E
#define ACK2_REG        	0x203F
#define ART1_REG        	0x2040
#define ART2_REG        	0x2041
#define SYN1_REG        	0x2042
#define SYN2_REG        	0x2043
#define AFEP1_REG       	0x2044
#define AFEP2_REG       	0x2045
#define DSSS1_REG       	0x2046
#define DSSS2_REG       	0x2047
#define DSSS3_REG       	0x2048
#define DSSS4_REG       	0x2049
#define DSSS5_REG       	0x204A
#define DSSS6_REG       	0x204B
#define CDET1_REG       	0x204C
#define CDET2_REG       	0x204D
#define DC_REG          	0x204E
#define RX1_REG         	0x204F
#define RX2_REG         	0x2050
#define RX3_REG         	0x2051
#define RX4_REG         	0x2052
#define CODE1_REG       	0x2053
#define CODE2_REG       	0x2054
#define ADC1_REG        	0x2055
#define ADC2_REG        	0x2056
#define ID0_REG         	0x2057
#define ID1_REG         	0x2058
#define ID2_REG         	0x2059
#define ID3_REG         	0x205A
#define ID4_REG         	0x205B
#define ID5_REG         	0x205C
#define ID6_REG         	0x205D
#define ID7_REG         	0x205E
#define DID3_REG        	0x205F
#define DID2_REG        	0x2060
#define DID1_REG        	0x2061
#define DID0_REG        	0x2062
#define NONCE0_REG			0x2063
#define NONCE1_REG			0x2064
#define NONCE2_REG			0x2065
#define NONCE3_REG			0x2066
#define NONCE4_REG			0x2067
#define NONCE5_REG			0x2068
#define NONCE6_REG			0x2069
#define NONCE7_REG      	0x206A
#define NONCE8_REG      	0x206B
#define NONCE9_REG      	0x206C
#define NONCE10_REG			0x206D
#define NONCE11_REG			0x206E
#define NONCE12_REG			0x206F
#define NONCE13_REG			0x2070
#define NONCE14_REG			0x2071
#define NONCE15_REG			0x2072
#define NONCE16_REG			0x2073
#define AESK0_REG			0x2074
#define AESK1_REG			0x2075
#define AESK2_REG			0x2076
#define AESK3_REG			0x2077
#define AESK4_REG			0x2078
#define AESK5_REG			0x2079
#define AESK6_REG       	0x207A
#define AESK7_REG       	0x207B
#define AESK8_REG       	0x207C
#define AESK9_REG       	0x207D
#define AESK10_REG			0x207E
#define AESK11_REG			0x207F
#define AESK12_REG			0x2080
#define AESK13_REG			0x2081
#define AESK14_REG			0x2082
#define AESK15_REG			0x2083
#define AESD0_REG			0x2084
#define AESD1_REG			0x2085
#define AESD2_REG			0x2086
#define AESD3_REG			0x2087
#define AESD4_REG			0x2088
#define AESD5_REG			0x2089
#define AESD6_REG       	0x208A
#define AESD7_REG       	0x208B
#define AESD8_REG       	0x208C
#define AESD9_REG       	0x208D
#define AESD10_REG			0x208E
#define AESD11_REG			0x208F
#define AESD12_REG			0x2090
#define AESD13_REG			0x2091
#define AESD14_REG			0x2092
#define AESD15_REG			0x2093
#define TXPKT0_REG      	0x2094
#define TXPKT1_REG      	0x2095
#define RXPKT0_REG      	0x2096
#define RXPKT1_REG      	0x2097
#define AESCTL0_REG     	0x2098
#define AESCTL1_REG     	0x2099
#define ADCCTL_REG      	0x209A
#define ADCAVG1_REG     	0x209B
#define ADCAVG2_REG     	0x209C
#define ADCAVG3_REG     	0x209D
#define TMRINV_REG      	0x209E
#define TMRCTL_REG      	0x209F
#define INTIE_REG       	0x20A0
#define EXT1_REG        	0x20A1
#define EXT2_REG        	0x20A2
#define EXT3_REG        	0x20A3
#define EXT4_REG        	0x20A4
#define EXT5_REG        	0x20A5
#define EXT6_REG        	0x20A6
#define PA_REG          	0x20A7
#define TCKG0_REG       	0x20A8
#define TCKG1_REG       	0x20A9
#define DAHOLD_REG      	0x20AA
#define INTF_REG			0x20AB
#define CLKSEL_REG      	0x20AC
#define RFAR_REG			0x20AD
#define VTS_REG				0x20AE
#define ECC_CFG0_REG		0x20AF
#define ECC_CFG1_REG		0x20B0
#define ECC_CFG2_REG		0x20B1
#define ECC_CFG3_REG		0x20B2
#define ECC_CFG4_REG		0x20B3
#define USB_CFG0_REG		0x20B4
#define USB_CFG1_REG		0x20B5
#define MTLH_REG	    	0x20B6
#define PM3_no_reset_REG	0x20B7
#define TX5_REG				0x20B8
#define TX6_REG				0x20B9
#define TMRDLY_REG			0x20BA
#define ASK0_REG			0x20BB
#define ASK1_REG			0x20BC
#define ASK2_REG			0x20BD
#define ASK3_REG			0x20BE
#define ASK4_REG			0x20BF
#define ASK5_REG			0x20C0
#define ASK6_REG			0x20C1
#define MRLH_REG			0x20C2
#define MSLH_REG			0x20C3  
#define BERT_REG			0x20FE
#define BIST_REG			0x20FF
#define TX_FIFO             0x2100  // 0x2100~0x217F = 128 bytes
#define RX_FIFO             0x2180  // 0x2180~0x21FF = 128 bytes

//--------------------------------------------------------------------------
// Strobe and Reset Command
//--------------------------------------------------------------------------

/* RF Strobe command */
#define CMD_SLEEP       0x80    // SLEEP mode
#define CMD_IDLE        0x90    // IDLE mode
#define CMD_STBY        0xA0    // Standby mode
#define CMD_PLL         0xB0    // PLL mode
#define CMD_RX          0xC0    // RX mode
#define CMD_TX          0xD0    // TX mode

/* RF Reset command */
#define RF_RST          0x80    //RF reset
#define TXPOINT_RST     0x40    //TX FIFO address pointrt reset
#define RXPOINT_RST     0x20    //RX FIFO address pointer reset
#define TRXFIFO_RST     0x08  //TRX FIFO data reset


#endif