#define _MAIN_C_
/*********************************************************************
**  Device:   A9139-F6
**  File:     main.c
**  Target:   A9139-F6
**  Tools:    ICE
**  Updated:  2019-06-10
**  Description:
**  This file is a sample code for your reference.
**
**  Copyright (C) 2019 AMICCOM Corp.
**
*********************************************************************/
#include <absacc.h>
#include "..\include\define.h"
#include "..\include\A9139F6.h"
#include "..\include\LIB_A9139F6.h"
#include "..\include\LIB_Uti.h"
#include "..\include\gpio.h"

/********************************************************************
** Configure Select
*********************************************************************/

#define A40_DR10K_IFBW50K     0
#define A40_DR10K_IFBW100K    1
#define A40_DR50K_IFBW50K     2
#define A40_DR100K_IFBW100K   3
#define A40_DR150K_IFBW150K   4
#define A40_DR250K_IFBW250K   5
#define A40_DR500K_IFBW500K   6
#define A40_DR9K6_IFBW192K    7

/********************************************************************/
/*            User Select RF BAND & Data Rate
=====================================================================
*             note: Data Rate 150Kbps Xtal = 19.2MHz
*                   Data Rate 250Kbps Xtal = 16MHz
*                   Data Rate 500Kbps Xtal = 16MHz
*/

#define SelectConfig          A40_DR50K_IFBW50K  // User define

/*********************************************************************/

#if   SelectConfig ==  A40_DR10K_IFBW50K   ||\
      SelectConfig ==  A40_DR10K_IFBW100K  ||\
      SelectConfig ==  A40_DR50K_IFBW50K   ||\
      SelectConfig ==  A40_DR100K_IFBW100K ||\
			SelectConfig ==  A40_DR9K6_IFBW192K

//crystal=12.8MHz
#define BaudRate_9600_BRG_M   82
#define BaudRate_9600_BRG_F   5
#define BaudRate_19200_BRG_M  40
#define BaudRate_19200_BRG_F  11
#define BaudRate_38400_BRG_M  19
#define BaudRate_38400_BRG_F  13
#define BaudRate_57600_BRG_M  12
#define BaudRate_57600_BRG_F  14
#define BaudRate_115200_BRG_M 5
#define BaudRate_115200_BRG_F 15

#elif		SelectConfig == A40_DR150K_IFBW150K

//crystal=19.2MHz
#define BaudRate_9600_BRG_M   124
#define BaudRate_9600_BRG_F   1
#define BaudRate_19200_BRG_M  61
#define BaudRate_19200_BRG_F  8
#define BaudRate_38400_BRG_M  30
#define BaudRate_38400_BRG_F  4
#define BaudRate_57600_BRG_M  19
#define BaudRate_57600_BRG_F  13
#define BaudRate_115200_BRG_M 9
#define BaudRate_115200_BRG_F 7

#elif		SelectConfig == A40_DR250K_IFBW250K ||\
				SelectConfig == A40_DR500K_IFBW500K

//crystal=16MHz
#define BaudRate_9600_BRG_M   103
#define BaudRate_9600_BRG_F   3
#define BaudRate_19200_BRG_M  51
#define BaudRate_19200_BRG_F  1
#define BaudRate_38400_BRG_M  25
#define BaudRate_38400_BRG_F  1
#define BaudRate_57600_BRG_M  16
#define BaudRate_57600_BRG_F  6
#define BaudRate_115200_BRG_M 7
#define BaudRate_115200_BRG_F 11

#endif
/*********************************************************************
**  I/O Declaration
*********************************************************************/
#define CKO     P1_3    //CKO
#define GIO1    P0_7    //GIO1
#define GIO2    P1_2    //GIO2

/*********************************************************************
**  Constant Declaration
*********************************************************************/
#define TIMEOUT     100   //100ms
#define t0hrel      1067  //1ms @12.8MHz

#define Enable_RFINT_TWWS   0x01
#define Enable_RFINT_WTR    0x02
#define Enable_RFINT_FPF    0x04
#define Enable_RFINT_FSTNC  0x08

#define ADC_SOURCE_CLK_DIV_4	(0x00<<5)
#define ADC_SOURCE_CLK_DIV_8	(0x01<<5)
#define ADC_SOURCE_CLK_DIV_16	(0x02<<5)
#define ADC_SOURCE_CLK_DIV_32	(0x03<<5)

#define AVSEL									(0x00<<6)
#define AVSEL_NO_AVERAGE			(0x00<<4)
#define AVSEL_AVERAGE_2				(0x01<<4)
#define AVSEL_AVERAGE_4				(0x02<<4)
#define AVSEL_AVERAGE_8				(0x03<<4)

#define MVSEL									(0x01<<6)
#define MVSEL_AVERAGE_8				(0x00<<2)
#define MVSEL_AVERAGE_16			(0x01<<2)
#define MVSEL_AVERAGE_32			(0x02<<2)
#define MVSEL_AVERAGE_64			(0x03<<2)

#define ADC_NO_AVERAGE			  (0x00<<1)
#define ADC_2_TIMES_AVERAGE	  (0x01<<1)
#define ADC_4_TIMES_AVERAGE	  (0x02<<1)
#define ADC_8_TIMES_AVERAGE	  (0x03<<1)
#define ADC_16_TIMES_AVERAGE	(0x04<<1)
#define ADC_32_TIMES_AVERAGE	(0x05<<1)
#define ADC_64_TIMES_AVERAGE	(0x06<<1)
#define ADC_128_TIMES_AVERAGE	(0x07<<1)

/*********************************************************************
**  Function I/O Declaration
*********************************************************************/
//CKO
#define DCK             0
#define RCK             1
#define FPF             2
#define PF8M            9
#define ROSC            10
//GIO1
#define ARCWTR          0
#define EOAC            1
#define TMEO            2
#define PMDO            3
#define TWOR            4
#define TRXD            7
#define VPOAK           12
#define FMTDO           15
//GIO2
#define ARCWTR_GPIO2    0
#define EOAC_GPIO2      1
#define TMEO_GPIO2      2
#define PMDO_GPIO2      3
#define TWWS_GPIO2      4
#define TRXD_GPIO2      7
#define VPOAK_GPIO2     12


#define TIMERPERIOD    	400
#define TIMERTIMEOUT    300
#define TIMERINIT       60
#define REPORTIMER      60
#define CHANNELNUM      4
#define MAXCHANNEL      400

/*********************************************************************
**  Global Variable Declaration
*********************************************************************/
Uint16  data    timer;
Uint8   data    HoppingCount;
bit     data    Hoppingflag;
Uint8   data    TimeoutFlag;
Uint16  idata   RxCnt;
Uint32  idata   Err_ByteCnt;
Uint32  idata   Err_BitCnt;
Uint16  idata   TimerCnt0;
Uint8   data    *Uartptr;
Uint8   data    UartSendCnt;
Uint8   data    CmdBuf[11];
Uint8   xdata   tmpbuf[64];
Uint8   data    Flag_MASTER;
Uint8   data    RF_FLAG;
Uint8		data		Timeout_Cnt;
Uint8		data    Slot_TX;
Uint8		idata		f_Timeout;
Uint8   idata   Flag_FirstLink;
Uint16	idata 	Err_Loss;
Uint16  idata   Err_Frame;
Uint16  idata   channel;

const Uint8 code BitCount_Tab[16] = {0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4};
const Uint8 code ID_Tab[8] = {0xE7, 0x75, 0xC5, 0x8C, 0xC7, 0x33, 0x45, 0x33}; //ID code
const Uint8 code PN9_Tab[] =
{
  0xFF, 0x83, 0xDF, 0x17, 0x32, 0x09, 0x4E, 0xD1,
  0xE7, 0xCD, 0x8A, 0x91, 0xC6, 0xD5, 0xC4, 0xC4,
  0x40, 0x21, 0x18, 0x4E, 0x55, 0x86, 0xF4, 0xDC,
  0x8A, 0x15, 0xA7, 0xEC, 0x92, 0xDF, 0x93, 0x53,
  0x30, 0x18, 0xCA, 0x34, 0xBF, 0xA2, 0xC7, 0x59,
  0x67, 0x8F, 0xBA, 0x0D, 0x6D, 0xD8, 0x2D, 0x7D,
  0x54, 0x0A, 0x57, 0x97, 0x70, 0x39, 0xD2, 0x7A,
  0xEA, 0x24, 0x33, 0x85, 0xED, 0x9A, 0x1D, 0xE0
};  // This table are 64bytes PN9 pseudo random code.


#if   SelectConfig ==  A40_DR10K_IFBW50K   ||\
      SelectConfig ==  A40_DR10K_IFBW100K  ||\
      SelectConfig ==  A40_DR50K_IFBW50K   ||\
      SelectConfig ==  A40_DR100K_IFBW100K ||\
			SelectConfig ==  A40_DR9K6_IFBW192K

//User defined Frequency, 12.8MHz
const unsigned char code Freq_Tab[9] =
{
  //PLL2  PLL3  PLL4
  0x87, 0x50, 0x14,  //freq 433.001MHz
  0x87, 0x50, 0x14,  //freq 433.001MHz
  0x87, 0x50, 0x14,  //freq 433.001MHz
};

#elif		SelectConfig == A40_DR150K_IFBW150K

//User defined Frequency, 19.2MHz
const Uint8 code Freq_Tab[9] =
{
  0x5A, 0x35, 0x62,  //freq 433.001MHz
  0x5A, 0x35, 0x62,  //freq 433.001MHz
  0x5A, 0x35, 0x62,  //freq 433.001MHz
};

#elif		SelectConfig == A40_DR250K_IFBW250K ||\
				SelectConfig == A40_DR500K_IFBW500K 
//User defined Frequency, 16MHz
const Uint8 code Freq_Tab[9] =
{
  0x6C, 0x40, 0x10,  //freq 433.001MHz
  0x6C, 0x40, 0x10,  //freq 433.001MHz
  0x6C, 0x40, 0x10,  //freq 433.001MHz
};
#endif

//User defined AES_CCM
const Uint8 code CCM_FIFO[23]={0x10, 0x90, 0x50, 0xD0, 0x30, 0xB0, 0x70, 0xF0,
                               0x08, 0x88, 0x48, 0xC8, 0x28, 0xA8, 0x68, 0xE8,
                               0x18, 0x98, 0x58, 0xD8, 0x38, 0xB8, 0x78};

const Uint8 code AES_KEY_IN[16]={0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};

const Uint8 code NONCE[4][17]={
    {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x04}, //level 4
    {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x05}, //level 5
    {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x06}, //level 6
    {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x07}  //level 7
};

/*********************************************************************
**  function Declaration
*********************************************************************/
void InitMCU ( void );
void Timer0ISR ( void );
void UART0ISR ( void );
void RFISR ( void );
void InitTimer0 ( void );
void InitUART0_SBRG ( void );
void WriteID ( void );
void WriteFIFO ( void );
void RxPacket ( void );
void Err_State ( void );
void PM1 ( void );
void PM2 ( void );
void PM3 ( void );
void RCOSC_Cal ( Uint8 ConfigType );
void RTC_Enable ( Uint8 ConfigType );
void WOR_enable_by_preamble ( Uint8 ConfigType );
void WOR_enable_by_sync ( Uint8 ConfigType );
void WOR_enable_by_carrier ( Uint8 ConfigType );
void WOT_enable ( Uint8 ConfigType );
void TWOR_enable ( Uint8 ConfigType );
void TMR_enable ( Uint8 ConfigType );
void SAR_ADC_12B ( Uint8 ConfigType );
void RSSI_measurement ( Uint8 ConfigType );
void TXPower ( Uint8 TBG, Uint8 TDC, Uint8 PAC, Uint8 ConfigType );
void FIFOLength ( Uint16 FEP, Uint8 ConfigType );
void TXModeSel ( Uint8 MODE, Uint8 ConfigType );
void TXModulation ( Uint8 MODE, Uint8 ConfigType );
void GPIO1_Setting ( Uint8 GIO1S, Uint8 G1I, Uint8 G1OE, Uint8 ConfigType );
void GPIO2_Setting ( Uint8 GIO2S, Uint8 G2I, Uint8 G2OE, Uint8 ConfigType );
void CKO_Setting ( Uint8 CKOS, Uint8 CKOI, Uint8 CKOE, Uint8 ConfigType );
void AES_CCM_mode ( Uint8 ConfigType );
void ACK_mode ( Uint8 ConfigType );
void ACK_FIFO_mode ( Uint8 ConfigType );
void UserRegister(void);
void SETGPIO ( Uint8 gio1s, Uint8 gio2s, Uint8 ckos, Uint8 iosw, Uint8 ConfigType);
void Clr_timer(void);
/*********************************************************************
* main loop
*********************************************************************/
void main ( void )
{
  Uint8 Temp;         
	Config = SelectConfig;
  PCON &= ~0x10;  //PWE=0
	
  InitMCU();
  InitTimer0();
  InitUART0_SBRG();
	Delay10ms ( 1 );
	InitCrystalCL(28); //CrystalCL 0x1C(20pF)
	Temp = RFLIB_InitRF ( SelectConfig );
	if(Temp)
		{ Err_State(); }
  
	if ( P3_5 ) //P3_5=1, master
  { Flag_MASTER = 1; }
  else  //P3_5=0, slave
  { Flag_MASTER = 0; }

	//User define
	WriteID();
	UserRegister();
	RFLIB_WriteReg(SYCK2_REG, 0x30);	// Data rate = Data rate / ( 24 + 1 )				
	channel=0;
	RFLIB_SetCH(0); //430.001MHz
	/* 
		How to set RF channel
		RF base Freq : 430.001MHz
		Channel step : 0.1MHz
		SetCH        : 10
		Range        : 430.001MHz ~ 434.701MHz
		RF Frequency = RF base Freq + (Channel step * SetCH) = 430.001MHz + ( 0.1MHz * 10 ) = 431.001MHz 
 */
	
  if ( Flag_MASTER )
  {
		timer=0;
		TR0=1;  //Timer0 on
		EA=1;   //enable interrupt
		Slot_TX=0;
		WriteFIFO();  //write data to tx fifo
		SETGPIO ( GPIO1_FMTDO, GPIO1_ARCWTR, CKO_DCK, ENABLE, SelectConfig );
    while ( 1 )
    {
			if(Slot_TX)
			{
				Slot_TX = 0;
				RFLIB_StrobeCmd ( CMD_TX );
				Delay10us(1);
				while(GIO2);
				channel=channel+100;
				if(channel==MAXCHANNEL)
				{
					channel=0;
				}
				RFLIB_SetCH(channel);
			}
    }
  }
  else
  {
		TR0=1;  //Timer0 on
		EA=1;   //enable interrupt
    Flag_FirstLink=1;
		RxCnt = 0;
		Err_Loss=0;
		Err_Frame=0;
		Err_BitCnt = 0;
		f_Timeout = 0;
		RFLIB_StrobeCmd(CMD_PLL);	
		HoppingCount=0;
		Hoppingflag=0;
    while ( 1 )
    {
			if(timer==1&&HoppingCount==0)
			{
						
				SETGPIO ( GPIO1_ARCWTR, GPIO2_RXD, CKO_DCK, ENABLE, SelectConfig );
				RFLIB_StrobeCmd(CMD_STBY);
				RFLIB_StrobeCmd(CMD_RX);
				SETGPIO ( GPIO1_FSYNC, GPIO2_RXD, CKO_DCK, ENABLE, SelectConfig );
				Delay10us(1);
				f_Timeout=0;
				while((GIO1==0) && (f_Timeout==0));
				if (f_Timeout)
				{
					f_Timeout = 0;

					if (Flag_FirstLink)
					{
						Err_Loss = 0;
						while(GIO1==0);

						Clr_timer();
						SETGPIO ( GPIO1_ARCWTR, GPIO2_RXD, CKO_DCK, ENABLE, SelectConfig );
						while(GIO1);

						Flag_FirstLink = 0;
						HoppingCount=0;
					}
					else
					{
							Hoppingflag=1;
							RFLIB_StrobeCmd(CMD_PLL);
							SETGPIO ( GPIO1_ARCWTR, GPIO2_RXD, CKO_DCK, ENABLE, SelectConfig );
							channel=channel+100;
							if(channel==MAXCHANNEL)
							{
								Err_Loss++;
								channel=0;
							}
							RFLIB_SetCH(channel);
					}
				}
				else
				{
					Hoppingflag=0;
					Clr_timer();
					SETGPIO ( GPIO1_ARCWTR, GPIO2_RXD, CKO_DCK, ENABLE, SelectConfig );
					while(GIO1);
				  RFLIB_StrobeCmd(CMD_PLL);
					RxPacket();
				}
			}
    }
  }
}
/************************************************************************
**  Clr_timer
************************************************************************/
void Clr_timer(void)
{
	TR0 = 0;
	TH0 = (65536-t0hrel-5)>>8; // Reload Timer0 high byte,low byte
	TL0 = 65536-t0hrel-5;

	timer=TIMERINIT;//8
	TR0 = 1;
}
/************************************************************************
**  Timer0ISR
************************************************************************/
void Timer0ISR ( void ) interrupt 1
{
		TR0=0;
    TF0 = 0; // Clear Timer0 interrupt
    TH0 = (65536-t0hrel)>>8; // Reload Timer0 high byte,low byte
    TL0 = 65536-t0hrel;
		TR0=1;
    timer++;
	if (timer>TIMERPERIOD)//600
	{
    	timer=0;
			HoppingCount=HoppingCount+1;
	}
	if(HoppingCount==CHANNELNUM||Hoppingflag==1)
		HoppingCount=0;
	
	if (timer>TIMERTIMEOUT)//10
		f_Timeout=1;		

    if (timer==2)
    	Slot_TX=1;

//    TimerCnt++;
//    if (TimerCnt >=500)
//    {
//    	TimerCnt = 0;
//    	Flag_Report=1;
//    }
}

/************************************************************************
**  UART0ISR
************************************************************************/
void UART0ISR ( void ) interrupt 4 using 3
{
  if ( TI )
  {
    TI = 0;
    UartSendCnt--;

    if ( UartSendCnt != 0 )
    {
      Uartptr++;
      SBUF = *Uartptr;
    }
  }

  if ( RI )
  {
    RI = 0;
  }
}

/************************************************************************
**  RFISR
************************************************************************/
void RFISR ( void ) interrupt 10
{
  EIF |= 0x08;  //clear flag
  RF_FLAG = 0;

  //RFLIB_WriteReg(TMRCTL_REG, 0xE1); //TMRCKS=128Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xE3); //TMRCKS=64Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xE5); //TMRCKS=32Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xE7); //TMRCKS=16Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xE9); //TMRCKS=8Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xEB); //TMRCKS=4Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xED); //TMRCKS=2Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xEF); //TMRCKS=1Hz
}

/************************************************************************
**  InitMCU
************************************************************************/
void InitMCU ( void )
{
  P0    = 0xFF;
  P0OE  = 0xFF;
  P0PUN = 0xFF;
  P0WUN = 0xFF;

  P1    = 0xFF;
  P1OE  = 0xFF;
  P1PUN = 0xFF;
  P1WUN = 0xFF;

  P2    = 0xFF;
  P2OE  = 0xFF;
  P2PUN = 0xFF;
  P2WUN = 0xFF;

  P3    = 0xFF;
  P3OE  = 0xDE;   //P3.0=RXD(input), P3.5=Mode Select(input)
  P3PUN = 0xDE;
  P3WUN = 0xFF;

  IOSEL   = IOSEL | 0x08;   //enable RF GPIO pin : GIO1=P0.7, GIO2=P1.2, CKO=P1.3
}

/************************************************************************
**  InitTimer0
************************************************************************/
void InitTimer0 ( void )
{
  TR0   = 0;
  TMOD  = ( TMOD & 0xF0 ) | 0x01; //Timer0 mode1
  TH0   = ( 65536 - t0hrel ) >> 8;  //setup Timer0 high byte, low byte
  TL0   = 65536 - t0hrel;
  TF0   = 0;          //Clear any pending Timer0 interrupts
  ET0   = 1;          //Enable Timer0 interrupt
}

/************************************************************************
**  InitUART0_SBRG
************************************************************************/
void InitUART0_SBRG ( void )
{
  Uint16 BRG_M;
  Uint8 BRG_F;

  IOSEL  = IOSEL | 0x01;    //enable UART pin

  BRG_M  = BaudRate_19200_BRG_M;
  BRG_F  = BaudRate_19200_BRG_F;
  SBRG0H = ( BRG_M >> 4 ) | 0x80;
  SBRG0L = ( BRG_M << 4 ) & 0x00F0 | BRG_F;

  SCON   = 0x40;
  REN    = 1;
  ES     = 1;   //enable uart0 interrupt
}


/************************************************************************
**  RFLIB_WriteID
************************************************************************/
void WriteID ( void )
{
  Uint8 i;
  Uint8 d[8];

  //Write ID
  for ( i = 0; i < 8; i++ )
  { RFLIB_WriteReg ( ( ID0_REG + i ), ID_Tab[i] ); }

  //Read ID
  for ( i = 0; i < 8; i++ )
  { d[i] = RFLIB_ReadReg ( ID0_REG + i ); }

  //for check
  for ( i = 0; i < 8; i++ )
  {
    if ( d[i] != ID_Tab[i] )
    {
      Err_State();
    }
  }
}

/************************************************************************
**  WriteFIFO
************************************************************************/
void WriteFIFO ( void )
{
  Uint16 i;

  RFLIB_ResetCmd ( TXPOINT_RST ); //TX FIFO address pointer reset

  for ( i = 0; i < 64; i++ )
  {
    RFLIB_WriteReg ( ( i + TX_FIFO ), PN9_Tab[i] );
  }
}

/*********************************************************************
** UserRegister
*********************************************************************/
void UserRegister(void)
{	
	// FIFO length
	RFLIB_WriteReg(FIFO1_REG, 0x3F);	// 64Bytes
	
	//TX power
	RFLIB_WriteReg(TX1_REG, 0x57);	// b'[5:6]: TDC
	RFLIB_WriteReg(TX4_REG, 0x3F);	// b'[0:5]: TBG, b'[6:7]: PAC

	// Battery Decter
	RFLIB_WriteReg(PM2_REG, 0x40);	// PM2_REG
	
	RFLIB_WriteReg(MODEC2_REG, 0xA0);	
	RFLIB_WriteReg(GPIO1_REG, 0x3D);	// GIO1_REG
	RFLIB_WriteReg(GPIO2_REG, 0x09);	// GIO2_REG
	RFLIB_WriteReg(CKO2_REG, 0x01);	// CKO2_REG
	RFLIB_WriteReg(CODE1_REG, 0x00);	// CODE1_REG
	RFLIB_WriteReg(CODE2_REG, 0x0D);	// CODE2_REG
	RFLIB_WriteReg(ADC1_REG, 0x7C);	// ADC1_REG
	
	RFLIB_WriteReg(SYCK2_REG, 0x00);	// Data rate = Data rate / ( SDR + 1 ) 
}

/************************************************************************
**  RxPacket
************************************************************************/
void RxPacket ( void )
{
  Uint16 i;
  Uint8 recv;
  Uint8 tmp,err;
	err=0;
	
  RxCnt++;

  RFLIB_ResetCmd ( RXPOINT_RST ); //RX FIFO address pointer reset

  for ( i = 0; i < 64; i++ )
  {
    tmpbuf[i] = RFLIB_ReadReg ( i + RX_FIFO );
  }

  for ( i = 0; i < 64; i++ )
  {
    recv = tmpbuf[i];
    tmp = recv ^ PN9_Tab[i];

    if ( tmp != 0 )
    {
      Err_ByteCnt++;
      Err_BitCnt += ( BitCount_Tab[tmp >> 4] + BitCount_Tab[tmp & 0x0F] );
			err=1;
    }
  }
	if (err)//packet error
	Err_Frame++;
	
		_nop_();
}

/************************************************************************
**  Err_State
************************************************************************/
void Err_State ( void )
{
  //ERR display
  //Error Proc...
  //...
	
                     /* Watchdog reset */
//                   EIE|=0x20;
//         
//                   CKCON=0x00;     
//                   CKCON=0x40;
//                   CKCON=0x80;
//                   CKCON=0xC0;
                     
//                   EA=0;
//                   
//                   TA=0xAA;//timed access register procedure
//                   TA=0x55;
//                   WDCON=0x02;
                     /* Watchdog reset End*/
  while ( 1 );
}

/************************************************************************
**  PM1
************************************************************************/
void PM1 ( void )
{
  //all pin as input and pull high
  IOSEL   = 0x00;

  P0OE  = 0x00;
  P0PUN = 0x00;
  P0    = 0xFF;
  P0WUN = 0xFF;

  P1OE  = 0x00;
  P1PUN = 0x00;
  P1    = 0xFF;
  P1WUN = 0xFF;

  P2OE  = 0x00;
  P2PUN = 0x00;
  P2    = 0xFF;
  P2WUN = 0xFF;

  P3OE  = 0x00;
  P3PUN = 0x00;
  P3    = 0xFF;
  P3WUN = 0xFF;

	//SRAM1K always exist(0x0000 ~ 0x03FF). If you want add SRAM memory keeps, you can use setting as below:
	RFLIB_WriteReg ( 0x20B7,  0x45 ); // SRAM7K OFF
//  RFLIB_WriteReg ( 0x20B7,  0x25 ); // SRAM7K ON (0x0400 ~ 0x1FFF)

	
  RFLIB_WriteReg ( EXT2_REG, 0x0A );
  RFLIB_WriteReg ( PM1_REG, 0xE8 );
  RFLIB_StrobeCmd ( CMD_SLEEP );
  PCONE = 0x01;
  PCON = 0x02;
  _nop_();
  _nop_();

  while ( PCON & 0x02 );
}

/************************************************************************
**  PM2
************************************************************************/
void PM2 ( void )
{
  //all pin as input and pull high
  IOSEL   = 0x00;

  P0OE  = 0x00;
  P0PUN = 0x00;
  P0    = 0xFF;
  P0WUN = 0xFF;

  P1OE  = 0x00;
  P1PUN = 0x00;
  P1    = 0xFF;
  P1WUN = 0xFF;

  P2OE  = 0x00;
  P2PUN = 0x00;
  P2    = 0xFF;
  P2WUN = 0xFF;

  P3OE  = 0x00;
  P3PUN = 0x00;
  P3    = 0xFF;
  P3WUN = 0xFF;

	//SRAM1K always exist(0x0000 ~ 0x03FF). If you want add SRAM memory keeps, you can use setting as below:
	RFLIB_WriteReg ( 0x20B7,  0x45 ); // SRAM7K OFF
//  RFLIB_WriteReg ( 0x20B7,  0x25 ); // SRAM7K ON (0x0400 ~ 0x1FFF)
	
  RFLIB_WriteReg ( EXT2_REG, 0x06 );
  RFLIB_WriteReg ( PM1_REG, 0xE9 );
  RFLIB_StrobeCmd ( CMD_SLEEP );
  PCONE = 0x01;
  PCON = 0x02;
  _nop_();
  _nop_();

  while ( PCON & 0x02 );
}


/************************************************************************
**  PM3
************************************************************************/
void PM3 ( void )
{
  //all pin as input and pull high
  IOSEL   = 0x00;

  P0OE  = 0x00;
  P0PUN = 0x00;
  P0    = 0xFF;
  P0WUN = 0xFF;

  P1OE  = 0x00;
  P1PUN = 0x00;
  P1    = 0xFF;
  P1WUN = 0xFF;

  P2OE  = 0x00;
  P2PUN = 0x00;
  P2    = 0xFF;
  P2WUN = 0xFF;

  P3OE  = 0x00;
  P3PUN = 0x00;
  P3    = 0xFF;
  P3WUN = 0xFF;

  RFLIB_WriteReg ( EXT2_REG, 0x05 );
  RFLIB_WriteReg ( PM1_REG, 0xE9 );
  RFLIB_StrobeCmd ( CMD_SLEEP );
  PCONE = 0x09;
  PCON = 0x02;
  _nop_();
  _nop_();

  while ( PCON & 0x02 );
}

/************************************************************************
**  RCOSC_Cal
************************************************************************/
void RCOSC_Cal ( Uint8 ConfigType )
{
  Uint16 tmp;
  Uint16 TGNUM = 195; //crystal=12.8M
  //Uint16 TGNUM=244; //crystal=16M
  //Uint16 TGNUM=293; //crystal=19.2M

  RFLIB_StrobeCmd ( CMD_STBY );

  //RFLIB_WriteReg(CKO2_REG, 0x53); //ROSC=32.768KHz
  //RFLIB_WriteReg(CKO2_REG, 0x43); //WCK=4.096KHz

  RFLIB_WriteReg ( RCOSC4_REG, ( TGNUM >> 8 ) );  //TGNUM
  RFLIB_WriteReg ( RCOSC5_REG, TGNUM );   //TGNUM

  RFLIB_WriteReg ( WOR4_REG, A9139Config[ConfigType][WOR4_REG - CONFIG] | 0x04 ); //enable RC OSC

  while ( 1 )
  {
    RFLIB_WriteReg ( RCOSC1_REG, A9139Config[ConfigType][RCOSC1_REG - CONFIG] | 0x01 ); //set ENCAL=1 to start RC OSC CAL

    do
    {
      tmp = RFLIB_ReadReg ( RCOSC1_REG );
    }
    while ( tmp & 0x01 );

    tmp = ( RFLIB_ReadReg ( RCOSC1_REG ) & 0xF0 );  //read NUMLH[11:8]
    tmp <<= 4;
    tmp = tmp + RFLIB_ReadReg ( RCOSC2_REG );   //read NUMLH[7:0]

    if ( ( tmp > ( TGNUM - 10 ) ) && ( tmp < ( TGNUM + 10 ) ) ) //NUMLH[11:0]=TGNUM+-10
    {
      break;
    }
  }
}

/************************************************************************
**  RTC_Enable
************************************************************************/
void RTC_Enable (  Uint8 ConfigType )
{
  IOSEL |= 0x10;  //enable RTCIOS
  P3OE  &= 0x3F;
  P3PUN &= 0x3F;

  //RFLIB_WriteReg(CKO2_REG, 0x53);   //ROSC(RTC)

  RFLIB_WriteReg ( RCOSC2_REG, A9139Config[ConfigType][RCOSC2_REG - CONFIG] | 0x88 ); //RTCPUN=1
	RFLIB_WriteReg ( RCOSC3_REG, A9139Config[ConfigType][RCOSC3_REG - CONFIG] | 0x12 ); //MRCT=18

  RFLIB_WriteReg ( RCOSC4_REG, A9139Config[ConfigType][RCOSC4_REG - CONFIG] | 0x80 ); //RTCS=1
  RFLIB_WriteReg ( RCOSC1_REG, A9139Config[ConfigType][RCOSC1_REG - CONFIG] | 0x82 ); //RCOT[2]=1, MAN = 1
	RFLIB_WriteReg ( USB_CFG0_REG, A9139Config[ConfigType][USB_CFG0_REG - CONFIG] | 0x80);    //INTXC32K=1
	RFLIB_WriteReg ( WOR4_REG, A9139Config[ConfigType][WOR4_REG - CONFIG] | 0x04 ); //enable RC OSC

  //delay for RTC stabilized
  //...
  //...
}

/************************************************************************
**  WOR_enable_by_preamble
************************************************************************/
void WOR_enable_by_preamble ( Uint8 ConfigType )
{
  RCOSC_Cal ( ConfigType ); //internal RC OSC CAL
  //RTC_Enable();   //use external RC OSC

  //RFLIB_WriteReg(CKO2_REG, 0x0B); //CKO=RCK
  //RFLIB_WriteReg(GPIO1_REG, (A9139Config[ConfigType][GPIO1_REG - CONFIG] & 0xC0) | 0x0D); //GPIO1=PMDO
  //RFLIB_WriteReg(GPIO1_REG, (A9139Config[ConfigType][GPIO1_REG - CONFIG] & 0xC0) | 0x11); //GPIO1=TWWS
  //RFLIB_WriteReg(GPIO2_REG, (A9139Config[ConfigType][GPIO2_REG - CONFIG] & 0xC0) | 0x01); //GPIO2=WTR

  //setup WOR Sleep time and Rx time
  //WOR Active Period = (WOR_AC[8:0]+1) x (1/32768), (30.5us ~ 15.6ms).
  //WOR Sleep  Period = (WOR_SL[9:0]+1) x (32/4096), (7.8ms ~ 7.99s).
  RFLIB_WriteReg ( WOR2_REG, 0x7F );    //WOR_SL[9:0]=127 : WOR Sleep  Period ~ 1s
  RFLIB_WriteReg ( WOR1_REG, 0x00 );
  RFLIB_WriteReg ( AFEP2_REG, 0x80 ); //WOR_AC[8:0]=256 : WOR Active Period ~ 7.84ms

  //WOR_HOLD : WOR hold RX setting when preamble/sync/carrier detected ok.
  //      [0]: No hold. [1]: Hold RX.
  //WOR_SEL: TWWS=1 setting.
  //      [0]:RX valid packet.  [1]: preamble/sync/carrier  detected ok.
  //WOR_RST: Shall be set to [1].
  RFLIB_WriteReg ( WOR4_REG, 0x0C );  //by preamble
  RFLIB_WriteReg ( WOR3_REG, 0x7E );  //WOR_SEL=1
  RFLIB_WriteReg ( DSSS4_REG, 0x03 ); //WOR_HOLD=1, WOR_RST=1

  RFLIB_StrobeCmd ( CMD_STBY );

  ERFINT = 1; //enable RF interrupt
  RFLIB_WriteReg ( INTIE_REG, Enable_RFINT_TWWS );  //enable TWWS INT

  RFLIB_WriteReg ( MODEC2_REG, A9139Config[ConfigType][MODEC2_REG - CONFIG] | 0x10 ); //enable WOR function

  PM2();

  //WOR_SEL=1 : preamble/sync/carrier detected ok -> TWWS go high -> wake up MCU
  //defined process by user
  //...
  //...
  //RFLIB_StrobeCmd(CMD_STBY);  //TWWS reset by strobe command.
}

/************************************************************************
**  WOR_enable_by_sync
************************************************************************/
void WOR_enable_by_sync ( Uint8 ConfigType )
{
  RCOSC_Cal ( ConfigType ); //internal RC OSC CAL
  //RTC_Enable();   //use external RC OSC

  //RFLIB_WriteReg(CKO2_REG, 0x0B); //CKO=RCK
  //RFLIB_WriteReg(GPIO1_REG, (A9139Config[ConfigType][GPIO1_REG - CONFIG] & 0xC0) | 0x05); //GPIO1=FSYNC
  //RFLIB_WriteReg(GPIO1_REG, (A9139Config[ConfigType][GPIO1_REG - CONFIG] & 0xC0) | 0x11); //GPIO1=TWWS
  //RFLIB_WriteReg(GPIO2_REG, (A9139Config[ConfigType][GPIO2_REG - CONFIG] & 0xC0) | 0x01); //GPIO2=WTR

  //setup WOR Sleep time and Rx time
  //WOR Active Period = (WOR_AC[8:0]+1) x (1/32768), (30.5us ~ 15.6ms).
  //WOR Sleep  Period = (WOR_SL[9:0]+1) x (32/4096), (7.8ms ~ 7.99s).
  RFLIB_WriteReg ( WOR2_REG, 0x7F );    //WOR_SL[9:0]=127 : WOR Sleep  Period ~ 1s
  RFLIB_WriteReg ( WOR1_REG, 0x00 );
  RFLIB_WriteReg ( AFEP2_REG, 0x80 ); //WOR_AC[8:0]=256 : WOR Active Period ~ 7.84ms

  //WOR_HOLD : WOR hold RX setting when preamble/sync/carrier detected ok.
  //      [0]: No hold. [1]: Hold RX.
  //WOR_SEL: TWWS=1 setting.
  //      [0]:RX valid packet.  [1]: preamble/sync/carrier  detected ok.
  //WOR_RST: Shall be set to [1].
  RFLIB_WriteReg ( WOR4_REG, 0x04 );  //by sync
  RFLIB_WriteReg ( WOR3_REG, 0x3E );  //WOR_SEL=0
  RFLIB_WriteReg ( DSSS4_REG, 0x03 ); //WOR_HOLD=1, WOR_RST=1

  RFLIB_StrobeCmd ( CMD_STBY );

  ERFINT = 1; //enable RF interrupt
  RFLIB_WriteReg ( INTIE_REG, Enable_RFINT_TWWS );  //enable TWWS INT

  RFLIB_WriteReg ( MODEC2_REG, A9139Config[ConfigType][MODEC2_REG - CONFIG] | 0x10 ); //enable WOR function

  PM2();

  //WOR_SEL=0 : RX valid packet -> TWWS go high -> wake up MCU
  //check data
  RxPacket();
  RFLIB_StrobeCmd ( CMD_STBY ); //TWWS reset by strobe command.
}

/************************************************************************
**  WOR_enable_by_carrier
************************************************************************/
void WOR_enable_by_carrier ( Uint8 ConfigType )
{
  RCOSC_Cal ( ConfigType ); //internal RC OSC CAL
  //RTC_Enable();   //use external RC OSC

  //RFLIB_WriteReg(CKO2_REG, 0x0B); //CKO=RCK
  //RFLIB_WriteReg(GPIO1_REG, (A9139Config[ConfigType][GPIO1_REG - CONFIG] & 0xC0) | 0x09); //GPIO1=CD
  //RFLIB_WriteReg(GPIO1_REG, (A9139Config[ConfigType][GPIO1_REG - CONFIG] & 0xC0) | 0x11); //GPIO1=TWWS
  //RFLIB_WriteReg(GPIO2_REG, (A9139Config[ConfigType][GPIO2_REG - CONFIG] & 0xC0) | 0x01); //GPIO2=WTR

  //setup WOR Sleep time and Rx time
  //WOR Active Period = (WOR_AC[8:0]+1) x (1/32768), (30.5us ~ 15.6ms).
  //WOR Sleep  Period = (WOR_SL[9:0]+1) x (32/4096), (7.8ms ~ 7.99s).
  RFLIB_WriteReg ( WOR2_REG, 0x7F );    //WOR_SL[9:0]=127 : WOR Sleep  Period ~ 1s
  RFLIB_WriteReg ( WOR1_REG, 0x00 );
  RFLIB_WriteReg ( AFEP2_REG, 0x80 ); //WOR_AC[8:0]=256 : WOR Active Period ~ 7.84ms

  //WOR_HOLD : WOR hold RX setting when preamble/sync/carrier detected ok.
  //      [0]: No hold. [1]: Hold RX.
  //WOR_SEL: TWWS=1 setting.
  //      [0]:RX valid packet.  [1]: preamble/sync/carrier  detected ok.
  //WOR_RST: Shall be set to [1].
  RFLIB_WriteReg ( WOR4_REG, 0x04 );
  RFLIB_WriteReg ( WOR3_REG, 0x7F );  //by carrier, WOR_SEL=1
  RFLIB_WriteReg ( DSSS4_REG, 0x03 ); //WOR_HOLD=1, WOR_RST=1

  RFLIB_WriteReg ( ADC1_REG, A9139Config[ConfigType][ADC1_REG - CONFIG] | 0x80 ); //ARSSI=1
  RFLIB_WriteReg ( ADC2_REG, 0x6E );    //RTH=110 (-100dBm)

  RFLIB_StrobeCmd ( CMD_STBY );

  ERFINT = 1; //enable RF interrupt
  RFLIB_WriteReg ( INTIE_REG, Enable_RFINT_TWWS );  //enable TWWS INT

  RFLIB_WriteReg ( MODEC2_REG, A9139Config[ConfigType][MODEC2_REG - CONFIG] | 0x10 ); //enable WOR function

  PM2();

  //WOR_SEL=1 : preamble/sync/carrier detected ok -> TWWS go high -> wake up MCU
  //defined process by user
  //...
  //...
  //RFLIB_StrobeCmd(CMD_STBY);  //TWWS reset by strobe command.
}

/************************************************************************
**  WOT_enable
************************************************************************/
void WOT_enable ( Uint8 ConfigType )
{
  RCOSC_Cal ( ConfigType ); //internal RC OSC CAL
  //RTC_Enable();   //use external RC OSC

  //RFLIB_WriteReg(CKO2_REG, 0x03); //CKO=DCK
  //RFLIB_WriteReg(GPIO1_REG, (A9139Config[ConfigType][GPIO1_REG - CONFIG] & 0xC0) | 0x05); //GPIO1=FSYNC
  //RFLIB_WriteReg(GPIO1_REG, (A9139Config[ConfigType][GPIO1_REG - CONFIG] & 0xC0) | 0x11); //GPIO1=TWWS
  //RFLIB_WriteReg(GPIO2_REG, (A9139Config[ConfigType][GPIO2_REG - CONFIG] & 0xC0) | 0x01); //GPIO2=WTR

  //setup WOT Sleep time
  //WOT Sleep Period = (WOR_SL[9:0]+1) x (32/4096), (7.8ms ~ 7.99s).
  RFLIB_WriteReg ( WOR2_REG, 0x7F );    //WOR_SL[9:0]=127 : WOT Sleep  Period ~ 1s
  
	//setup WOT Active time
	//WOT Active time = (WOR_AC[8:0]+1) x (1/32768), (30.5us ~ 15.6ms).
  RFLIB_WriteReg ( WOR1_REG, 0x0E );		//WOR_AC[8:0] shall be more than 0x0E

  //WOR_RST: Shall be set to [1].
  RFLIB_WriteReg ( DSSS4_REG, 0x02 ); //WOR_RST=1

  RFLIB_StrobeCmd ( CMD_STBY );
  WriteFIFO();  //write data to TX FIFO

  ERFINT = 1; //enable RF interrupt
  RFLIB_WriteReg ( INTIE_REG, Enable_RFINT_TWWS );  //enable TWWS INT

  RFLIB_WriteReg ( PIN1_REG, A9139Config[ConfigType][PIN1_REG - CONFIG] | 0x04 );   //select WOT function
  RFLIB_WriteReg ( MODEC2_REG, A9139Config[ConfigType][MODEC2_REG - CONFIG] | 0x10 ); //enable WOT function

  PM2();

  //defined process by user
  //...
  //...
  //RFLIB_WriteReg(MODEC2_REG, A9139Config[ConfigType][MODEC2_REG - CONFIG] & 0xEF);     //disable WOT function
  //RFLIB_StrobeCmd(CMD_STBY);  //TWWS reset by strobe command.
}

/************************************************************************
**  TWOR_enable
************************************************************************/
void TWOR_enable ( Uint8 ConfigType )
{
  RCOSC_Cal ( ConfigType ); //internal RC OSC CAL
  //RTC_Enable();   //use external RC OSC

  //RFLIB_WriteReg(GPIO1_REG, (A9139Config[ConfigType][GPIO1_REG - CONFIG] & 0xC0) | 0x11); //GPIO1=TWWS
  //RFLIB_WriteReg(GPIO2_REG, (A9139Config[ConfigType][GPIO2_REG - CONFIG] & 0xC0) | 0x11); //GPIO2=TWWS

  //WOR_AC Period = (WOR_AC[8:0]+1) x (1/4096), (244us ~ 125ms).
  //WOR_SL Period = (WOR_SL[9:0]+1) x (32/4096), (7.8ms ~ 7.99s).
  //Note : TWOR function which enables this device to output a periodic square wave from GPIO to a MCU
  //                        _____     _____   _____
  //    TWWS signal : _____|   |_____|   |_____|   |
  //                    1s    1s    1s    1s    1s    1s
  //
  //    TWWS interrupt are activated by a rising edge.
  //    first time, interrupt interval=1s
  //    other times, interrupt interval=2s

  RFLIB_WriteReg ( WOR2_REG, 0x7F );    //WOR_SL[9:0]=127 : WOR_SL Period ~ 1s
  RFLIB_WriteReg ( WOR1_REG, 0x19 );
  RFLIB_WriteReg ( AFEP2_REG, 0xC0 ); //WOR_AC[8:0]=409 : WOR_AC Period ~ 0.1s

  ERFINT = 1; //enable RF interrupt
  RFLIB_WriteReg ( INTIE_REG, Enable_RFINT_TWWS );  //enable TWWS INT

  //RFLIB_WriteReg(WOR4_REG, A9139Config[ConfigType][WOR4_REG - CONFIG] | 0x05);  //enable TWOR function by WOR_AC
  RFLIB_WriteReg ( WOR4_REG, A9139Config[ConfigType][WOR4_REG - CONFIG] | 0x07 ); //enable TWOR function by WOR_SL

  PM2();

  //TWWS go high -> wake up MCU
  //defined process by user
  //...
  //...
  //RFLIB_WriteReg(WOR4_REG, A9139Config[ConfigType][WOR4_REG - CONFIG] | 0x04);  //disable TWOR function
}

/************************************************************************
**  TMR_enable
************************************************************************/
void TMR_enable ( Uint8 ConfigType )
{
  RCOSC_Cal ( ConfigType ); //internal RC OSC CAL
  //RTC_Enable();   //use external RC OSC

  //TMR Period = (1 / (2 ^ (7-TMRCKS[2:0]) ) ) * (TMRINV + 1)
  //ex : TMRCKS=7, TMRINV=0
  //     TMR Period = (1 / (2 ^ (7-7) ) ) * (0+1) = 1/1 = 1s

  RFLIB_WriteReg ( TMRINV_REG, 0x00 );  //TMRINV=0

  //RFLIB_WriteReg(TMRCTL_REG, 0xE1); //TMRCKS=128Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xE3); //TMRCKS=64Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xE5); //TMRCKS=32Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xE7); //TMRCKS=16Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xE9); //TMRCKS=8Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xEB); //TMRCKS=4Hz
  //RFLIB_WriteReg(TMRCTL_REG, 0xED); //TMRCKS=2Hz
  RFLIB_WriteReg ( TMRCTL_REG, 0xEF );  //TMRCKS=1Hz

  ERFINT = 1; //enable RF interrupt

  PM2();
  //PM3();

  //defined process by user
  //...
  //...
}

/************************************************************************
**  SAR_ADC_12B
************************************************************************/
void SAR_ADC_12B ( Uint8 ConfigType )
{
  Uint16 MVADC_value;
  //Uint8 tmp;

  ADCCH = 0x09;      //ADC input = P2.0
  P2OE   = P2OE & ~0x01;  //P2.0 as input
  P2PUN  = P2PUN | 0x01;  //close pull-high

  //ADCCH = 0x0B;    //ADC input = P2.1
  //P2OE   = P2OE & ~0x02;  //P2.1 as input
  //P2PUN  = P2PUN | 0x02;  //close pull-high

  //ADCCH = 0x0D;    //ADC input = P2.2
  //P2OE   = P2OE & ~0x04;  //P2.2 as input
  //P2PUN  = P2PUN | 0x04;  //close pull-high

  //ADCCH = 0x0F;    //ADC input = P2.3
  //P2OE   = P2OE & ~0x08;  //P2.3 as input
  //P2PUN  = P2PUN | 0x08;  //close pull-high

  //ADCCH = 0x01;    //ADC input = P3.2
  //P3OE   = P3OE & ~0x04;  //P3.2 as input
  //P3PUN  = P3PUN | 0x04;  //close pull-high

  //ADCCH = 0x03;    //ADC input = P3.3
  //P3OE   = P3OE & ~0x08;  //P3.3 as input
  //P3PUN  = P3PUN | 0x08;  //close pull-high

  //ADCCH = 0x05;    //ADC input = P3.4
  //P3OE   = P3OE & ~0x10;  //P3.4 as input
  //P3PUN  = P3PUN | 0x10;  //close pull-high

  //ADCCH = 0x07;    //ADC input = P3.5
  //P3OE   = P3OE & ~0x20;  //P3.5 as input
  //P3PUN  = P3PUN | 0x20;  //close pull-high

  RFLIB_WriteReg ( ADC1_REG, A9139Config[ConfigType][ADC1_REG - CONFIG] | 0x02 );   //set XADS
  RFLIB_WriteReg ( ADCAVG1_REG, A9139Config[ConfigType][ADCAVG1_REG - CONFIG] | 0x82 ); //ENADC=1 & enable ADC INT

  ERFINT = 1; //enable RF interrupt

  while ( 1 )
  {
    RFLIB_WriteReg ( ADCCTL_REG, (A9139Config[ConfigType][ADCCTL_REG - CONFIG] & 0x80) | 0x01 | ADC_32_TIMES_AVERAGE | ADC_SOURCE_CLK_DIV_4); //Start ADC conversion, single mode
		//MVS[2:0]: [000]:ADC_NO_AVERAGE, [001]: ADC_2_TIMES_AVERAGE, [010]:ADC_4_TIMES_AVERAGE, [011]:ADC_8_TIMES_AVERAGE,
		//[100]:ADC_16_TIMES_AVERAGE, [101]:ADC_32_TIMES_AVERAGE, [110]:ADC_64_TIMES_AVERAGE, [111]:ADC_128_TIMES_AVERAGE
		
		//CKS[1:0]: [00]:ADC_SOURCE_CLK_DIV_4, [01]:ADC_SOURCE_CLK_DIV_8, [10]:ADC_SOURCE_CLK_DIV_16, [11]:ADC_SOURCE_CLK_DIV_32
		
		//The time of A/D conversion 1 times is :4*32*2^CKS[1:0]/Fxtal
		//ex:CKS[1:0]=[00],Fxtal=12.8MHz, the time of A/D conversion 1 times is :4*32*2^0/12.8M =10us.
		//If ADC_32_TIMES_AVERAGE, the time of A/D conversion is 10us * 32 = 320us.

		//for single mode use
//		do{
//			tmp = RFLIB_ReadReg(ADCCTL_REG);  //wait ADC conversion complete
//		}while(tmp & 0x01);
		
    RF_FLAG = 1;		
    while ( RF_FLAG );    //wait ADC conversion complete

    //read MVADC
    MVADC_value = ( RFLIB_ReadReg ( ADCAVG1_REG ) & 0xF0 );
    MVADC_value <<= 4;
    MVADC_value = MVADC_value + RFLIB_ReadReg ( ADCAVG2_REG );
  }
}

/************************************************************************
**  RSSI_measurement
************************************************************************/
void RSSI_measurement ( Uint8 ConfigType )
{
  Uint8 tmp;
//	Uint16 tmp1;

  RFLIB_StrobeCmd ( CMD_STBY );
	
  //RFLIB_WriteReg(GPIO1_REG, (A9139Config[ConfigType][GPIO1_REG - CONFIG] & 0xC0) | 0x05); //GPIO1=FSYNC
  //RFLIB_WriteReg(GPIO2_REG, (A9139Config[ConfigType][GPIO2_REG - CONFIG] & 0xC0) | 0x01); //GPIO2=WTR
	
	//RFLIB_WriteReg(ADC1_REG, (A9139Config[ConfigType][ADC1_REG - CONFIG] & 0x0F) | 0x80 | AVSEL | AVSEL_NO_AVERAGE); //ARSSI=1, AVSEL(RADC=0),ADC average is set by AVSEL
	//AVSEL[1:0]: [00]:AVSEL_NO_AVERAGE, [01]:AVSEL_AVERAGE_2, [10]:AVSEL_AVERAGE_4, [11]:AVSEL_AVERAGE_8,
	
	RFLIB_WriteReg(ADC1_REG, (A9139Config[ConfigType][ADC1_REG - CONFIG] & 0x33) | 0x80 | MVSEL | MVSEL_AVERAGE_8);  //ARSSI=1, MVSEL(RADC=1),ADC average is set by MVSEL
	//MVSEL[1:0]: [00]:MVSEL_AVERAGE_8, [01]:MVSEL_AVERAGE_16, [10]:MVSEL_AVERAGE_32, [11]:MVSEL_AVERAGE_64,

	//The ADC clock is :PF8M/1.6, ADC clock periods is 1/ADC clock
	//Each RSSI measurning time is 30 * ADC clock periods * 8(MVSEL_AVERAGE_8) + (1/PF8M)*8*5
	//IF data rate=9.6Kbps,PF8M=6.14M. Data rate=250Kbps,PF8M=8M. Else PF8M=6.4M.
	//ex:Fxtal=12.8MHz,data rate=100Kbps, the ADC clock is :6.4M/1.6=4MHz
	//Each RSSI measurning time is 30 * (1/4M) * 8 + (1/6.4M) * 8 * 5 = 66.25us
	
  ERFINT = 1; //enable RF interrupt
  RFLIB_WriteReg ( INTIE_REG, Enable_RFINT_FSTNC ); //enable FSYNC interrupt

  RFLIB_StrobeCmd ( CMD_RX );

  RF_FLAG = 1;

  while ( RF_FLAG )   //Stay in RX mode until receiving ID code(sync ok)
  {
    tmp = RFLIB_ReadReg ( ADC2_REG );     																					 //read RSSI value(environment RSSI)
		//tmp1 = ((RFLIB_ReadReg ( RX1_REG ) & 0x01) <<8 ) | RFLIB_ReadReg ( RX2_REG );  //read RSSI value(environment RSSI) ADCO[8:0] when AGC on
  }

  tmp = RFLIB_ReadReg ( ADC2_REG );    																							 //read RSSI value(wanted signal RSSI)
	//tmp1 = ((RFLIB_ReadReg ( RX1_REG ) & 0x01) <<8 ) | RFLIB_ReadReg ( RX2_REG );    //read RSSI value(wanted signal RSSI) ADCO[8:0] when AGC on

}

/************************************************************************
**  TXPower
**
**  TBG Range 0~63
**  TDC Range 0~3
**  PAC Range 0~3
************************************************************************/
void TXPower ( Uint8 TBG, Uint8 TDC, Uint8 PAC, Uint8 ConfigType )
{

  RFLIB_WriteReg ( TX1_REG, ( ( ( A9139Config[ConfigType][TX1_REG - CONFIG] ) & ~0x60 ) | ( ( TDC & 0x03 ) << 5 ) ) );
  RFLIB_WriteReg ( TX4_REG, ( ( ( A9139Config[ConfigType][TX4_REG - CONFIG] ) & ~0xFF ) | ( ( TBG & 0x3F ) << 0 ) | ( ( PAC & 0x03 ) << 6 ) ) );

}
/************************************************************************
**  FIFOLength
**
**  FEP Range 0~16384
************************************************************************/
void FIFOLength ( Uint16 FEP, Uint8 ConfigType )
{

  RFLIB_WriteReg ( FIFO1_REG, ( ( FEP - 1 ) & 0xFF ) );
  RFLIB_WriteReg ( VCB1_REG,  ( ( ( A9139Config[ConfigType][VCB1_REG - CONFIG] ) & ~0x3F ) | ( ( ( FEP - 1 ) & 0x3F00 ) >> 8 ) ) );
}
/************************************************************************
**  TXModeSel
**
**  MODE : 0 - DIRECT MODE ; 1 - FIFO MODE
************************************************************************/
void TXModeSel ( Uint8 MODE, Uint8 ConfigType )
{
  if ( MODE == DIRECT )
  { RFLIB_WriteReg ( MODEC2_REG,  A9139Config[ConfigType][MODEC2_REG - CONFIG] & ~0x20 ); }//FMS = 0
  else
  { RFLIB_WriteReg ( MODEC2_REG,  ( ( A9139Config[ConfigType][MODEC2_REG - CONFIG] & ~0x20 ) | ( 0x01 << 5 ) ) ); } //FMS = 1
}
/************************************************************************
**  TXModulation
**
**  MODE : 0 -DISABLE ; 1 - ENABLE
************************************************************************/
void TXModulation ( Uint8 MODE, Uint8 ConfigType )
{
  if ( MODE == ENABLE )
  { RFLIB_WriteReg ( TX1_REG,  ( ( A9139Config[ConfigType][TX1_REG - CONFIG] & ~0x10 ) | ( 0x01 << 4 ) ) ); } //TME = 1
  else
  { RFLIB_WriteReg ( TX1_REG,  A9139Config[ConfigType][TX1_REG - CONFIG] & ~0x10 ); }//TME = 0
}

/************************************************************************
**  GIO1
**
************************************************************************/
void GPIO1_Setting ( Uint8 GIO1S, Uint8 G1I, Uint8 G1OE, Uint8 ConfigType )
{
  RFLIB_WriteReg ( GPIO1_REG, ( ( ( A9139Config[ConfigType][GPIO1_REG - CONFIG] ) & ~0x3F ) | ( ( GIO1S & 0x0F ) << 2 )| ( ( G1I & 0x01 ) << 1 ) | ( ( G1OE & 0x01 ) << 0 ) ) );
}

/************************************************************************
**  GIO2
**
************************************************************************/
void GPIO2_Setting ( Uint8 GIO2S, Uint8 G2I, Uint8 G2OE, Uint8 ConfigType )
{
  RFLIB_WriteReg ( GPIO2_REG, ( ( ( A9139Config[ConfigType][GPIO2_REG - CONFIG] ) & ~0x3F ) | ( ( GIO2S & 0x0F ) << 2 )| ( ( G2I & 0x01 ) << 1 ) | ( ( G2OE & 0x01 ) << 0 ) ) );
}

/************************************************************************
**  CKO
**
************************************************************************/
void CKO_Setting ( Uint8 CKOS, Uint8 CKOI, Uint8 CKOE, Uint8 ConfigType )
{
  RFLIB_WriteReg ( CKO2_REG, ( ( ( A9139Config[ConfigType][CKO2_REG - CONFIG] ) & ~0x7E ) | ( ( CKOS & 0x0F ) << 3 )| ( ( CKOI & 0x01 ) << 2 ) | ( ( CKOE & 0x01 ) << 1 ) ) );
}

/************************************************************************
**  AES_CCM_mode
************************************************************************/
void AES_CCM_mode ( Uint8 ConfigType )
{
  Uint8 tmp, AES_CCM_FIFO_Length;
    Uint16 addr, i;

    addr = AESK0_REG;
    for(i=0; i<16; i++)
        RFLIB_WriteReg((addr + i), AES_KEY_IN[i]);

    addr = NONCE0_REG;
    for(i=0; i<17; i++)
        RFLIB_WriteReg((addr + i), NONCE[0][i]);    //CCM Nonce Setting level 4~7 => 0~3

    AES_CCM_FIFO_Length=23;

    RFLIB_WriteReg(TXPKT0_REG, 0x00);
    RFLIB_WriteReg(TXPKT1_REG, 0x03);               //TX_PACKET Setting=0~3

    ERFINT = 1; //enable RF interrupt
    RFLIB_WriteReg ( INTIE_REG, Enable_RFINT_WTR ); //enable WTR interrupt
    FIFOLength ( AES_CCM_FIFO_Length, ConfigType);

    if ( Flag_MASTER )
    {

        GPIO1_Setting(FMTDO ,DISABLE,ENABLE,ConfigType);//GIO1 FMTDO
        GPIO2_Setting(TMEO  ,DISABLE,ENABLE,ConfigType);//GIO2 TMEO
        CKO_Setting  (DCK   ,DISABLE,ENABLE,ConfigType);//CKO  DCK

        while(1)
        {
            RFLIB_WriteReg(RSTCTL_REG, TRXFIFO_RST);    //FIFO clear
            RFLIB_WriteReg(AESCTL0_REG, 0x21);          //CCM on

            RFLIB_WriteReg(RSTCTL_REG, TXPOINT_RST);    //TX FIFO address pointer reset
            for(i=0; i<AES_CCM_FIFO_Length; i++)
            {
                RFLIB_WriteReg((i + TX_FIFO), CCM_FIFO[i]);
            }

            RFLIB_WriteReg(AESCTL0_REG, 0xA1);          //AES-CCM Encode
            do{
                tmp = RFLIB_ReadReg(AESCTL1_REG);
                tmp &= 0x20;
            }while(tmp==0);

            RFLIB_StrobeCmd ( CMD_TX );
            RF_FLAG = 1;

            while ( RF_FLAG );    //wait transmit completed

            Delay10ms ( 1 );

        }
    }
    else
    {

        GPIO1_Setting(TRXD  ,DISABLE,ENABLE,ConfigType);//GIO1 TRXD
        GPIO2_Setting(EOAC  ,DISABLE,ENABLE,ConfigType);//GIO2 EOAC/FSYNC
        CKO_Setting  (DCK   ,DISABLE,ENABLE,ConfigType);//CKO  DCK

        while ( 1 )
        {
            RFLIB_WriteReg(RSTCTL_REG, TRXFIFO_RST);    //FIFO clear

            RFLIB_WriteReg(AESCTL0_REG, 0x21);          //CCM on

            RFLIB_StrobeCmd ( CMD_RX );
            RF_FLAG = 1;
            while ( RF_FLAG );    //wait receive completed

            RFLIB_WriteReg(AESCTL0_REG, 0x61);          //AES-CCM Decode
            do{
                tmp = RFLIB_ReadReg(AESCTL1_REG);
                tmp &= 0x20;
            }while(tmp==0);

            if(tmp != 0x20)                             //AES-CCM Decode Fail
            {
                Delay100us ( 1 );
                while(1);
            }

            Delay100us ( 1 );

        }

    }

}
/************************************************************************
**  ACK_mode
************************************************************************/
void ACK_mode ( Uint8 ConfigType )
{
    RFLIB_WriteReg(PM2_REG, A9139Config[ConfigType][PM2_REG - CONFIG] | 0x80);  //SPSS=1

    if ( Flag_MASTER )
    {

        RFLIB_WriteReg(ACK2_REG, 0x3F); //by event, fixed, ARC=15, EAR=1
        RFLIB_WriteReg(ART2_REG, 0x1E); //ACK format = Preamble + ID +Payload
                                        //ARD=30 : (4+4+64+2)x8x10us=5.92ms / 250us = 24
        RFLIB_WriteReg(ART1_REG, 0x09); //RND=9

        GPIO1_Setting(VPOAK     ,DISABLE,ENABLE,SelectConfig);//GIO1 VPOAK
        GPIO2_Setting(ARCWTR    ,DISABLE,ENABLE,SelectConfig);//GIO2 ARCWTR
        CKO_Setting  (DCK       ,DISABLE,ENABLE,SelectConfig);//CKO  DCK

        while(1)
        {
            WriteFIFO();

            RFLIB_StrobeCmd ( CMD_TX );

            while(GIO2==0);     //wait TX ready
            while(GIO2==1);     //wait transmit completed

            Delay100us ( 1 );
            RFLIB_StrobeCmd ( CMD_STBY );

            Delay10ms ( 1 );

        }

    }
    else
    {
        RFLIB_WriteReg(ACK2_REG, 0x01); //EAR=1

        GPIO1_Setting(TRXD  ,DISABLE,ENABLE,SelectConfig);//GIO1 TRXD
        GPIO2_Setting(EOAC  ,DISABLE,ENABLE,SelectConfig);//GIO2 EOAC/FSYNC
        CKO_Setting  (RCK   ,DISABLE,ENABLE,SelectConfig);//CKO  RCK

        while ( 1 )
        {
            RFLIB_StrobeCmd ( CMD_RX );

            while(GIO2==0);     //wait sync ok
            while(GIO2==1);     //wait receive completed.

            Delay100us ( 1 );  //delay 200us for TX ramp down

            RFLIB_StrobeCmd ( CMD_STBY );

        }
    }
}
/************************************************************************
**  ACK_FIFO_mode
************************************************************************/
void ACK_FIFO_mode ( Uint8 ConfigType )
{
    EA = 0;
    RFLIB_WriteReg(PM2_REG, A9139Config[ConfigType][PM2_REG - CONFIG] | 0x80);  //SPSS=1

    RFLIB_WriteReg(SYN1_REG, 0x03); //EAF=1, DC_SEL=1
    RFLIB_WriteReg(AFEP1_REG, 0x3F);//ACKFEP=63, ACK_FIFO length=64Byte

    WriteFIFO();

    if ( Flag_MASTER )
    {

        RFLIB_WriteReg(ACK2_REG, 0x3F); //by event, fixed, ARC=15, EAR=1
        RFLIB_WriteReg(ART2_REG, 0x1E); //ACK format = Preamble + ID +Payload
                                        //ARD=30 : (4+4+64+2)x8x10us=5.92ms / 250us = 24
        RFLIB_WriteReg(ART1_REG, 0x09); //RND=9

        CKO_Setting  (DCK  ,DISABLE,ENABLE,SelectConfig);//CKO  DCK

        while(1)
        {

        GPIO2_Setting(TMEO_GPIO2      ,DISABLE,ENABLE,SelectConfig);//GIO2 TMEO_GPIO2
        GPIO1_Setting(FMTDO      ,DISABLE,ENABLE,SelectConfig);//GIO1 FMTDO

            RFLIB_StrobeCmd ( CMD_TX );

            while(GIO2==0);     //wait TX ready
            while(GIO2==1);     //wait transmit completed

            GPIO2_Setting(TRXD_GPIO2   ,DISABLE,ENABLE,SelectConfig);//GIO2 TRXD
            GPIO1_Setting(EOAC       ,DISABLE,ENABLE,SelectConfig);//GIO1 EOAC/FSYNC

            while(GIO1==0);     //wait sync ok
            while(GIO1==1);     //wait receive completed

            GPIO2_Setting(VPOAK_GPIO2  ,DISABLE,ENABLE,SelectConfig);//GIO2 VPOAK

            Delay100us ( 1 );
            RFLIB_StrobeCmd ( CMD_STBY );
            Delay10ms ( 1 );

        }

    }
    else
    {
        RFLIB_WriteReg(ACK2_REG, 0x3F); //EAR=1

        CKO_Setting  (RCK   ,DISABLE,ENABLE,SelectConfig);//CKO  DCK

        while ( 1 )
        {

            GPIO2_Setting(EOAC  ,DISABLE,ENABLE,SelectConfig);//GIO2 EOAC/FSYNC
            GPIO1_Setting(TRXD  ,DISABLE,ENABLE,SelectConfig);//GIO1 TRXD

            RFLIB_StrobeCmd ( CMD_RX );

            while(GIO2==0);     //wait sync ok
            while(GIO2==1);     //wait receive completed.

            GPIO2_Setting(TMEO  ,DISABLE,ENABLE,SelectConfig);//GIO2 TMEO

            while(GIO2==0);     //wait TX ready

            GPIO1_Setting(FMTDO ,DISABLE,ENABLE,SelectConfig);//GIO1 FMTDO

            while(GIO2==1);     //wait transmit completed.

            GPIO2_Setting(VPOAK_GPIO2   ,DISABLE,ENABLE,SelectConfig);//GIO2 VPOAK

            Delay100us ( 1 );  //delay 200us for TX ramp down

            RFLIB_StrobeCmd ( CMD_STBY );
        }
    }
}

/************************************************************************
**  FCC
************************************************************************/
void FCC( void )
{
	Uint16 i;

	IOSEL |= 0x08;
 	EA=1;
  ERFINT=1;	
  RFLIB_WriteReg ( INTIE_REG, Enable_RFINT_WTR ); //enable WTR interrupt

	RFLIB_WriteReg(FIFO1_REG, 0x3F);	//64Byte
	RFLIB_ResetCmd(TXPOINT_RST);				//TX FIFO address pointer reset
	for(i=0; i<64; i++)
	{
		RFLIB_WriteReg((i + TX_FIFO), PN9_Tab[i]);
	}
	for(i=64; i<128; i++)
	{
		RFLIB_WriteReg((i + TX_FIFO), PN9_Tab[i-64]);
	}
	
	RFLIB_SetCH(0);

	RFLIB_StrobeCmd(CMD_PLL);

    while(1)
    {  	
			RF_FLAG=1;
    	RFLIB_StrobeCmd(CMD_TX);
			while(RF_FLAG==1);  		  

			Delay10us(1);
    }

	//Direct Tx or Rx
	// TXModeSel ( DIRECT , SelectConfig);	//direct mode
	//RFLIB_StrobeCmd(CMD_TX);
	//RFLIB_StrobeCmd(CMD_RX);
	//while(1);
}

void SETGPIO ( Uint8 gio1s, Uint8 gio2s, Uint8 ckos, Uint8 iosw, Uint8 ConfigType)
{

    RFLIB_WriteReg ( CKO2_REG,  (A9139Config[ConfigType][CKO2_REG  - CONFIG]) & 0x87 | ckos  << 3); /*CKO*/
    RFLIB_WriteReg ( GPIO1_REG, (A9139Config[ConfigType][GPIO1_REG - CONFIG]) & 0xC3 | gio1s << 2); /*GPIO1*/
    RFLIB_WriteReg ( GPIO2_REG, (A9139Config[ConfigType][GPIO2_REG - CONFIG]) & 0xC3 | gio2s << 2); /*GPIO2*/


    if ( iosw )
    {
        IOSEL = IOSEL |  0x08 ;  /*BBIOS = 1*/
        P0OE  = BIT7;//Output P0.7 GIO1
        P1OE  = BIT3 | BIT2; //Output P1.2 GIO2, P1.3 CKO
    }

    else
    {
        IOSEL = IOSEL & ~0x08;  /*BBIOS = 0*/
        P0OE  = P0OE & ~BIT7; //input P0.7
        P1OE  = P1OE & ~ ( BIT3 | BIT2 ); //input P1.2 GIO2, P1.3 CKO
    }
}